#include <stdio.h>

#include "tree.h"

#define  TREEWIDTH   3

#define  FULLSHOW    1


#if FULLSHOW
 #define SHOWTREE(a,b)      ( showtree((a), (b)) )
 #define NEUEZEILE          ( printf("\n") )
#else
 #define SHOWTREE(a,b)
 #define NEUEZEILE
#endif

t_tree *t1, *t2;




/******************************************************************************
 * Debug-Ausgabe-Routinen
 ******************************************************************************/

/*** gibt i Spaces aus ***/
void spaces(int i)
{
 while (i--)
   printf(" ");
}

/*** zeigt einen Tree an ***/
void showtree(t_tree * t, int s)
{
 int i;
 spaces(s);
 if (!t)
   {
      printf("<LEER>\n");
      return;
   }
 printf("|-%s (%.2f)\n", t->data.name, t->data.profit);

 for (i=0; i<t->data.childNodeCount; ++i)
   {
      showtree(t->next[i], s+TREEWIDTH);
   }
}


/*** gibt Fehler aus und beendet das Programm ***/
void printerror(int num)
{
 printf("\n---> Test %2d FEHLER!\n", num);
 exit(1); 
}

/*** gibt Test-okay aus ***/
void printokay(int num)
{
 printf("\n---> Test %2d erfolgreich!\n", num);
}



/******************************************************************************
 * TEST-FUNKTIONEN
 ******************************************************************************/

/************************************************/
/*** tree_makeEmptyTree ***/
/************************************************/
int test01(void)
{
 printf("\n[tree_makeEmptyTree]\n");

 t1 = tree_makeEmptyTree();
 SHOWTREE(t1, 0);

 return t1 == NULL;
}


/************************************************/
/*** tree_makeTree ***/
/************************************************/
int test02(void)
{
 int res=1;

 printf("\n[tree_makeTree]\n");

 t1 = tree_makeTree("hallo", 12.125f);    /* alles okay */
 SHOWTREE(t1, 0);
 if (t1 == NULL)  
   res=0;
 else 
   tree_destroyTree(t1);

 t1 = tree_makeTree("", 5000.4f);         /* ungltiger name */
 SHOWTREE(t1, 0);
 if (t1 != NULL)  res=0;
  
 t1 = tree_makeTree(NULL, 4.4f);          /* ungltiger name */
 SHOWTREE(t1, 0);
 if (t1 != NULL)  res=0;

 return res;
}


/************************************************/
/*** tree_destroyTree ***/
/************************************************/
int test03(void)
{
 int res=1;

 printf("\n[tree_destroyTree]\n");

 t1=NULL;                              
 t1=tree_destroyTree(t1);                 /* leerer tree */
 if (t1 != NULL)  res=0;


 t1 = tree_makeTree("test", 2.2f);
 t1 = tree_insertNode(t1, t1, "vorher", 1.0);
 t1 = tree_insertNode(t1, t1->next[0], "wowi", 23.00f);
 SHOWTREE(t1, 0);
 t1=tree_destroyTree(t1);                 /* voller tree */
 if (t1 != NULL)  res=0;
 SHOWTREE(t1, 0);
 NEUEZEILE;

 t1 = tree_makeTree("test", 2.2f);
 SHOWTREE(t1, 0);
 t1=tree_destroyTree(t1);                 /* tree mit nur wurzel */
 if (t1 != NULL)  res=0;
 SHOWTREE(t1, 0);

 return res;
}


/************************************************/
/*** tree_insertNode ***/
/************************************************/
/******************************************************************************
 * Fuegt einen einzelnen Knoten vor dem Element 'before' in einen Baum ein.
 * Ein Name darf nur einmal vorkommen !
 * Bei Fehler wird NULL zureuckgeliefert sonst der resultierende Tree
 * (Fehler wenn kein tree, keine before, kein name oder profit<0 oder name 
 * schon vorhanden)
 ******************************************************************************/

int test04(void)
{
 int res=1;
 t_tree * tmp;

 printf("\n[tree_insertNode]\n");

 /* leere Baum */
 printf("leere Baum\n");
 t1 = tree_makeEmptyTree();
 t1 = tree_insertNode(t1, t1, "hallo", 2.2f);
 if (t1!=NULL)  res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;

 /* Baum mit nur Wurzel */
 printf("Baum mit nur Wurzel\n");
 t2 = tree_makeTree("a b c d e", 0.1f);
 SHOWTREE(t2,0);
 NEUEZEILE;
 t1 = tree_insertNode(t2, t2, "neu", 2.2f);
 if (t1==NULL && t1!=t2)  res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* langer Baum, ber Wurzel einfgen */
 printf("langer Baum, ueber Wurzel einfuegen\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("drei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("vier", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("fuenf", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("sechs", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_insertNode(t1, t1, "NEUE NODE", 4.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* langer Baum, an Zweig einfgen */
 printf("langer Baum, an Zweig einfuegen\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("drei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("vier", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("fuenf", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("sechs", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_insertNode(t1, t1->next[0], "NEUE NODE", 4.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);


 /* normaler Baum, before nicht drinnen */
 printf("normaler Baum, before nicht drinnen\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("drei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("vier", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("fuenf", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("sechs", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t2 = tree_makeTree("anderer baum", 1.1f);
 tmp = tree_insertNode(t1, t2, "NEUE NODE", 4.0f);
 if (tmp != NULL)  res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);
 t2 = tree_destroyTree(t2);


 /* normaler Baum, Name schon vorhanden */
 printf("normaler Baum, Name schon vorhanden\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("drei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("vier", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("fuenf", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("sechs", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t2 = tree_insertNode(t1, t1, "zwei", 4.0f);
 if (t2 != NULL)  res=0;
 SHOWTREE(t2,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);


 return res;
}


/************************************************/
/*** tree_replaceNode ***/
/************************************************/
/******************************************************************************
 * Ersetzt eine Node durch eine neue mit den angegebenen Parametern.
 * Die Referenzen der alten Node werden uebernommen.
 * Ein Name darf nur einmal vorkommen !
 * Achtet darauf, den Speicher der ersetzten Node ordentlich freizugeben.
 * Bei Fehler wird NULL zureuckgeliefert. sonst der resultierende Tree
 * (Fehler wenn kein tree, keine node, kein name, profit<0, name schon 
 * vorhanden, node nicht im tree vorhanden)
 ******************************************************************************/

int test05(void)
{
 int res=1;
 t_tree * tmp;

 printf("\n[tree_replaceNode]\n");

 /* wurzel ohne referenzen ersetzen */
 printf("wurzel ohne referenzen ersetzen\n");
 t1 = tree_makeTree("mal wieder", 133.212f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_replaceNode(t1, t1, "neu", 0.1f);
 if (t1==NULL) res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* element anderen profit bei gleichem namen geben */
 printf("element anderen profit bei gleichem namen geben\n");
 t1 = tree_makeTree("eins", 133.212f);
 t1 = tree_insertNode(t1, t1, "zwei", 4.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_replaceNode(t1, t1, "zwei", 0.1f);
 SHOWTREE(t1,0);
 NEUEZEILE;

 t1 = tree_destroyTree(t1);


 /* wurzel mit referenzen ersetzen */
 printf("wurzel mit referenzen ersetzen\n");
 t1 = tree_makeTree("mal wieder", 133.212f);
 t1 = tree_insertNode(t1, t1, "hallo", 4.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_replaceNode(t1, t1, "neu", 0.1f);
 if (t1==NULL) res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* node ohne referenzen mitendrin ersetzen */
 printf("node ohne referenzen mitendrin ersetzen\n");
 t1 = tree_makeTree("juhu", 9.99999f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender1", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender2", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender3", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender4", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry1", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry2", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry3", 1.999f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_replaceNode(t1, t1->next[2], "zoidberg1", 45.0f);
 t1 = tree_replaceNode(t1, t1->next[0]->next[0], "zoidberg2", 45.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* node mit referenzen mitendrin ersetzen */
 printf("node mit referenzen mitendrin ersetzen\n");
 t1 = tree_makeTree("juhu", 9.99999f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender1", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender2", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender3", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender4", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry1", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry2", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry3", 1.999f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_replaceNode(t1, t1, "amy1", 45.0f);
 t1 = tree_replaceNode(t1, t1->next[0], "amy2", 45.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 /* node nicht im tree vorhanden */
 printf("node nicht im tree vorhanden\n");
 t1 = tree_makeTree("juhu", 9.99999f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender1", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender2", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender3", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender4", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry1", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry2", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry3", 1.999f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t2 = tree_makeTree("baum2",2.2f);
 tmp = tree_replaceNode(t1, t2, "ganz neu", 45.23f);
 if (tmp!=NULL) res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);
 t2 = tree_destroyTree(t2);

 /* name schon vorhanden */
 printf("name schon vorhanden\n");
 t1 = tree_makeTree("juhu", 9.99999f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender1", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender2", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender3", 1.999f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("bender4", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry1", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry2", 1.999f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("fry3", 1.999f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 tmp = tree_replaceNode(t1, t1, "fry3", 45.23f);
 if (tmp!=NULL) res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_removeNode ***/
/************************************************/
/******************************************************************************
 * Entfernt eine Node aus einem Baum. Alle Referenzen des zu entfernenden 
 * Baumes werden dem darueberliegendem Node angehaengt.
 * Liefert den resultierenden Baum oder NULL bei Fehler zurueck
 * (Fehler bei kein tree, kein node, oder node nicht im Baum)
 * Achtet darauf den Speicher der gelschten Node ordentlich freizugeben.
 ******************************************************************************/

int test06(void)
{
 int res=1;
 t_tree *tmp;

 printf("\n[tree_removeNode]\n");

 /* wurzel removen */
 printf("wurzel removen\n");
 t1 = tree_makeTree("homer", 1.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;
 tmp = tree_removeNode(t1,t1);
 if (tmp != NULL) res=0;
 t1 = tree_destroyTree(t1);

 /* element ohne referenzen removen */
 printf("element ohne referenzen removen\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("drei", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_removeNode(t1,t1->next[1]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_destroyTree(t1);

 /* element mit referenzen removen */
 printf("element mit referenzen removen\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("drei", 3.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("vier", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_removeNode(t1,t1->next[0]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_destroyTree(t1);

 /* element ohne referenz  removen (parent hat nur dieses element) */
 printf("element ohne referenz  removen (parent hat nur dieses element)\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("drei", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_removeNode(t1,t1->next[0]->next[0]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_destroyTree(t1);

 /* element mit referenz  removen (parent hat nur dieses element) */
 printf("element mit referenz  removen (parent hat nur dieses element)\n");
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("zwei", 2.0f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("drei", 3.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_removeNode(t1,t1->next[0]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_appendTree ***/
/************************************************/
/******************************************************************************
 * Hanegt einer Node in einem Baum einen Tree als zusaetzliche Referenz an.
 * Wenn diese Node vorher keine Referenz hat, hat sie danach eine, wenn sie
 * vorher eine hat, hat sie danach 2,usw.
 * Eine Node darf nur einmal vorkommen !
 * Gibt den resultierenden Baum oder bei Fehler NULL zurueck
 * (Fehler bei kein tree, keine node, kein insertTree, node nicht in tree)
 ******************************************************************************/

int test07(void)
{
 int res=1;

 printf("\n[tree_appendTree]\n");
 printf("\nkomplexen baum aufbauen");
 /* normalen baum aufbauen */
 t1 = tree_makeEmptyTree();
 t1 = tree_makeTree("ROOT", 10000.111f);
 t1 = tree_appendTree(t1, t1, tree_makeTree("bart", 2.2f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("2", 2.2f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("3", 2.2f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("4", 2.2f) );
 t1 = tree_appendTree(t1, t1, tree_makeTree("5", 2.2f) );
 t1 = tree_appendTree(t1, t1->next[0], tree_makeTree("6", 2.2f) );
 SHOWTREE(t1,0);
 NEUEZEILE;

 t2 = tree_makeEmptyTree();
 t2 = tree_makeTree("zwei", 20000.111f);
 t2 = tree_appendTree(t2, t2, tree_makeTree("mehr pack", 2.2f) );
 t2 = tree_appendTree(t2, t2->next[0], tree_makeTree("heiner", 2.2f) );
 t2 = tree_appendTree(t2, t2->next[0], tree_makeTree("peter", 2.2f) );
 t2 = tree_appendTree(t2, t2->next[0], tree_makeTree("klaus", 2.2f) );
 t1 = tree_appendTree(t1, t1->next[0], t2 );
 SHOWTREE(t1,0);
 NEUEZEILE;

 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_replaceTree ***/
/************************************************/
/******************************************************************************
 * Ersetzt eine Node durch den angegebenen 'replaceTree'. Die Referenzen 
 * des alten gehen verloren !
 * Achtet darauf, den Speicher der verlorengehenden Referenzen ordentlich
 * zu befreien.
 * Eine Node darf nur einmal vorkommen !
 * Gibt den resultierenden Baum oder bei Fehler NULL zurck
 * (Fehler bei kein tree, kein node, kein replaceTree, wenn node nicht in tree)
 ******************************************************************************/

int test08(void)
{
 int res=1;
 t_tree *tmp;
 
 printf("\n[tree_replaceTree]\n");

 /* node nicht in tree */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 2.2f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 3.3f) );
 t2 = tree_makeTree("aaa", 1.0f);
 t2 = tree_appendTree(t2,t2, tree_makeTree("bbb", 2.2f) );
 
 tmp = tree_replaceTree(t1, t2, t2);
 if (tmp!=NULL)   res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;
 
 /* replace tree ersetzt ein element ohne referenzen */
 t1 = tree_replaceTree(t1, t1->next[0], t2);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_removeTree ***/
/************************************************/
/******************************************************************************
 * Entfernt eine Node mitsamt seinen Referenzen aus einem Baum.
 * Gibt den resultierenden Baum zurueck oder NULL bei Fehler
 * (Fehler wenn kein tree, keine node, node nicht in tree)
 * Achtet darauf, den Speicher der verlorengehenden Referenzen ordentlich
 * zu befreien. 
 ******************************************************************************/

int test09(void)
{
 int res=1;

 printf("\n[tree_removeTree]\n");

 /* lsche gesamten baum von wurzel */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 2.2f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 3.3f) );
 t1 = tree_removeTree(t1, t1);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1!=NULL) res=0;

 /* lsche node mit referenzen */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 2.2f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 3.3f) );
 t1 = tree_removeTree(t1, t1->next[0]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_removeTree(t1, t1->next[0]);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1==NULL) res=0;
 t1 = tree_removeTree(t1, t1);
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (t1!=NULL) res=0;

 return res;
}


/************************************************/
/*** tree_inTree ***/
/************************************************/
/******************************************************************************
 * Prueft, ob ein Knoten in einem Baum vorhanden ist
 * Liefert 1 falls vorhanden, sonst 0, bei Fehler -1
 * (Fehler wenn kein tree, keine node)
 ******************************************************************************/

int test10(void)
{
 int res=1;

 printf("\n[tree_inTree]\n");

 /* knoten vorhanden und wurzel */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 2.2f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 3.3f) );
 if (tree_inTree(t1, t1) != 1)   res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;

 /* knoten mitendrin vorhanden */
 if (tree_inTree(t1, t1->next[0]) != 1)   res=0;

 /* knoten nicht da */
 t2 = tree_makeTree("doppelt", 34.34f);
 if (tree_inTree(t1, t2) != 0)   res=0;

 /* kein node */
 if (tree_inTree(t1, NULL) != -1)   res=0;

 t1 = tree_destroyTree(t1);
 t2 = tree_destroyTree(t2);

 return res;
}


/************************************************/
/*** tree_findName ***/
/************************************************/
/******************************************************************************
 * Prueft, ob ein Knoten mit dem Namen 'name' im Baum vorhanden ist und gibt 
 * ihn zurueck oder NULL bei Fehler
 * (Fehler wenn kein tree, kein name)
 ******************************************************************************/

int test11(void)
{
 int res=1;

 printf("\n[tree_findName]\n");

 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 2.2f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 3.3f) );

 /* name mitendrin vorhanden */
 if (tree_findName(t1, "zwei") != t1->next[0])   res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;

 /* name vorhanden und wurzel */
 if (tree_findName(t1, "eins") != t1)   res=0;


 /* ungltiger name */
 if (tree_findName(t1, "nicht da") != NULL)   res=0;

 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_updateProfit ***/
/************************************************/
/******************************************************************************
 * Erhoeht den Profit eines Nodes um den uebergebenen profit*(1-PROFIT_ANTEIL).
 * Simuliert also somit ein gemachtes Geschaeft eines Node, der PROFIT_ANTEIL
 * an die Nodes ueber ihm die ihn referenzieren abdruecken muss, d.h. er selbst
 * behaelt nur 1-PROFIT_ANTEIL des angegebenen Profits.
 * Aktualisiert also auch die Profits der Nodes darueber, je unter beruecksich-
 * tigung der PROFIT_ANTEIL Abgabe an Nodes darueber
 * Der oberste muss natuerlich nichts mehr abgeben, behaelt also dann 100%
 * Gibt den resuliterenden Baum zureuck oder NULL bei Fehler
 * (Fehler wenn kein tree, kein name, profit<0, name nicht in tree)
 ******************************************************************************/

int test12(void)
{
 int res=1;

 printf("\n[tree_updateProfit]\n");

 /* erhoehe profit fr wurzel */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 1.0f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 1.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 t1 = tree_updateProfit(t1, "eins", 9.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;

 /* erhoehe profit fr knoten mitendrin */
 t1 = tree_updateProfit(t1, "zwei", 4.0f);
 SHOWTREE(t1,0);
 NEUEZEILE;

 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_getNodeProfit ***/
/************************************************/
/******************************************************************************
 * Ermittelt den von einem Node erzielten Profit.
 * Gibt den Profit oder -1.0 bei Fehler zureuck
 * (Fehler wenn kein tree, kein name, name nicht in tree)
 ******************************************************************************/

int test13(void)
{
 int res=1;

 printf("\n[tree_getNodeProfit]\n");

 /* profit von wurzel */
 t1 = tree_makeTree("eins", 1.0f);
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 1.0f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 1.0f) );
 SHOWTREE(t1,0);
 NEUEZEILE;
 if (tree_getNodeProfit(t1, "eins") != t1->data.profit) res=0;

 /* profit von wert mitendrin */
 if (tree_getNodeProfit(t1, "zwei") != t1->next[0]->data.profit) res=0;

 /* profit von ungueltigem namen */
 if (tree_getNodeProfit(t1, "ungueltig") != -1.0f) res=0;

 t1 = tree_destroyTree(t1);

 return res;
}


/************************************************/
/*** tree_getTreeProfit ***/
/************************************************/
/******************************************************************************
 * Ermittelt den im gesamten Baum erzielten Profit.
 * Gibt den Profit oder -1.0 bei Fehler zureuck
 * (Fehler wenn kein tree)
 ******************************************************************************/

int test14(void)
{
 int res=1;

 printf("\n[tree_getTreeProfit]\n");

 /* profit von leerem baum */
 if (tree_getTreeProfit(NULL) != -1.0) res=0;

 /* profit von baum mit einem element */
 t1 = tree_makeTree("eins", 10.0f);
 if (tree_getTreeProfit(t1) != 10.0f) res=0;
 
 /* profit von normalen baum */
 t1 = tree_appendTree(t1,t1, tree_makeTree("zwei", 1.0f) );
 t1 = tree_appendTree(t1,t1, tree_makeTree("drei", 1.0f) );
 if (tree_getTreeProfit(t1) != 12.0f) res=0;
 SHOWTREE(t1,0);
 NEUEZEILE;

 t1 = tree_destroyTree(t1);

 return res;
}






/******************************************************************************
 * Hauptprogramm zum Testen der Lib
 ******************************************************************************/
int main (int argc, char * argv[])
{


 if (test01() )   printokay(1);
 else             printerror(1);

 if (test02() )   printokay(2);
 else             printerror(2);

 if (test03() )   printokay(3);
 else             printerror(3);

 if (test04() )   printokay(4);
 else             printerror(4);

 if (test05() )   printokay(5);
 else             printerror(5);

 if (test06() )   printokay(6);
 else             printerror(6);

 if (test07() )   printokay(7);
 else             printerror(7);

 if (test08() )   printokay(8);
 else             printerror(8);

 if (test09() )   printokay(9);
 else             printerror(9);

 if (test10() )   printokay(10);
 else             printerror(10);

 if (test11() )   printokay(11);
 else             printerror(11);

 if (test12() )   printokay(12);
 else             printerror(12);

 if (test13() )   printokay(13);
 else             printerror(13);

 if (test14() )   printokay(14);
 else             printerror(14);


 printf("\n\n--> memcount: %d", getmemcount() );

 printf("\n\n--> TEST ERFOLGREICH!\n\n");


 return 0;
}
 
